#ifndef CHAT_H
#define CHAT_H

#include <qcolor.h>
#include <qdialog.h>
#include <qmap.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qrect.h>
#include <qsplitter.h>
#include <qvariant.h>
#include <qvaluelist.h>

#include "emoticons.h"
#include "misc.h"
#include "userbox.h"

class Chat;
class QHBox;

typedef QValueList<Chat*> ChatList;

/**
	Klasa przechowujca informacje o wiadomoci,
	ktra ma si pojawi w oknie Chat
**/
class ChatMessage
{
	public:
	QString nick;
	QDateTime date;
	QDateTime sdate;
	QString unformattedMessage;
	bool isMyMessage;
	QColor backgroundColor;
	QColor textColor;
	
	/**
		Moe suy do przechowywania informacji o tym
		czy wiadomo bya potwierdzona, szyfrowana, ...
		W zamyle ma suy do okrelania czy jaki obrazek
		ma by dodawany do wiadomoci czy nie - jaki obrazek -
		ta informacja bdzie gdzie indziej
	**/
	QMap<QString, bool> attributes; 
	
	//inne atrybuty?
	//QMap<QString, QString> stringAttributes;
	
	bool needsToBeFormatted;
	
	/**
		Sformatowana wiadomo (razem z <p> lub <table>)
	**/
	QString message;
	/**
		Konstruktor ustawiajcy nick, niesformatowan wiadomo,
		czy wiadomo pochodzi od nas, dat wiadomoci,
		dat nadania wiadomoci przez serwer
	**/
	ChatMessage(const QString &nick, const QString &unformattedMessage, bool myMessage, QDateTime date, QDateTime sdate=QDateTime());
	/**
		Konstruktor ustawiajcy sformatowan wiadomo, kolor ta i liter wiadomoci
	**/
	ChatMessage(const QString &formattedMessage, const QColor &bgColor=Qt::white, const QColor &textColor=Qt::black);
};
/**
	Klasa zarzdzajca oknami Chat
**/
class ChatManager : public QObject
{
	Q_OBJECT

	private:
		ChatList Chats;
		int openPendingMsg(int index, ChatMessage &msg);
		
		struct ChatInfo
		{
			UinsList uins;
			QMap<QString, QVariant> map;
		};
		QValueList<ChatInfo> addons;

	public:	
		ChatManager(QObject* parent=NULL, const char* name=NULL);
		~ChatManager();
		/**
			Zwraca liste otwartych okien Chat.
		**/
		const ChatList& chats();
		/**
			Zwraca wskanik do okna z list uytkownikw uins
			@param uins lista uytkownikw
			@return wskanik do okna jeli istnieje w przeciwnym wypadku zwraca NULL
		**/
		Chat* findChatByUins(UinsList uins);
		/**
			zwraca warto wasnoci "name" okna okrelonego przez uins
			@param uins lista uytkownikw identyfikujca okno
			@param name nazwa wasnoci
			@return zwraca warto wasnoci jeli okrelone okno istnieje,
			jeli nie to tworzy tak wasno (ustawia na pust)
		**/
		QVariant& getChatProperty(const UinsList &uins, const QString &name);
		
	public slots:
		/**
			Slot informujcy o odebraniu wiadomoci
		**/
		void chatMsgReceived(UinsList senders,const QString& msg,time_t time,bool& grab);

		/**
			Otwiera nowe okno Chat z wymienionymi rozmowcami.
			Parametr time sluzy do sprawdzenia ile wiadomosci
			z historii ma sie pojawic w oknie.
		**/
		int openChat(UinsList senders,time_t time=0);		
		/**
			Wpisuje zakolejkowane wiadomoci do okna z uytkownikami "uins"
		**/
		void openPendingMsgs(UinsList uins);
		/**
			Wpisuje wszystkie zakolejkowane wiadomoci do odpowiednich okien
		**/
		void openPendingMsgs();
		/*
			Niebardzo rozumiem tej funkcji (czemu jest uin i uins)	
		*/
		void sendMessage(UinType uin,UinsList selected_uins);
		/**
			Zamyka wszystkie okna chat
		**/
		void closeAllWindows();
		/**
			Dodaje okno do managera
			@param chat wskanik do okna ktore chcemy doda
			@return zwraca numer naszego okna po zarejestrowaniu
		**/
		int registerChat(Chat* chat);
		/**
			Wyrejestrowuje okno z managera
			Zapisuje wasnoci okna
			wysya sygna chatDestroying i chatDestroyed
		**/
		void unregisterChat(Chat* chat);
		/**
			Odwiea tytuy wszystkich okien
		**/
		void refreshTitles();
		/**
			Odwiea tytuy okien ktre zawieraj uin
		**/
		void refreshTitlesForUin(UinType uin);
		/**
			Odwiea kolory i czcionki we wszystkich oknach
		**/
		void changeAppearance();
		/**
			pozwala przypisa okrelonemu czatowi (nawet jeeli on jeszcze nie istnieje)
			pewne wasnoci
		**/
		void setChatProperty(const UinsList &uins, const QString &name, const QVariant &value);
		
	signals:
		/**
		 	Sygna ten jest wysyany po utworzeniu nowego okna chat
			@param senders lista uin-w z ktrymi tworzymy nowy chat
		**/
		void chatCreated(const UinsList& senders);
		/**
		 	Sygna ten jest wysyany przed zamnkniciem okna chat
			@param senders lista uin-w z ktre wystpuj w tym chacie
		**/
		void chatDestroying(const UinsList& senders);
		/**
		 	Sygna ten jest wysyany po zamnkniciem okna chat
			@param senders lista uin-w z ktre wystpoway w tym chacie
		**/
		void chatDestroyed(const UinsList& senders);
		/**
			Sygna ten jest wysyaniy podczas kadej prby otwarcia nowego okna chcat
			nawet jeli ju taki istnieje
			@param senders lista uin-w
		**/
		void chatOpen(const UinsList& senders);
};

extern ChatManager* chat_manager;

class EmoticonSelector;
class ColorSelector;
/**
	Klasa umoliwiajca wpisywanie wiadomoci
**/
class CustomInput : public QMultiLineEdit
{
	Q_OBJECT

	protected:
		bool autosend_enabled;
		virtual void keyPressEvent(QKeyEvent *e);
		virtual void keyReleaseEvent(QKeyEvent* e);

	public:
		enum
		{
			KEY_BOLD,
			KEY_ITALIC,
			KEY_UNDERLINE
		};
		CustomInput(QWidget* parent = 0, const char* name = 0);

	public slots:
		void paste();
		/**
			Ustawia/wycza automatyczne wysyanie wiadomoci
		**/
		void setAutosend(bool);

	signals:
		/**
			Sygna zostaje wysany kiedy nacinito 
			klawisz wysania wiadomoci
		**/
		void sendMessage();
		/**
			Sygna zostaje wysany gdy nacinito specjalny klawisz (skrt)
			odpowiadajcy np KEY_BOLD
		**/
		void specialKeyPressed(int key);
		/**
			Dowolny przycisk klawiatury zosta nacinity.
			Przekazany zostaje take obiekt, ktry wywoa akcj - czyli this.
		**/
		void keyPressed(QKeyEvent* e, CustomInput* sender);
		/**
			Dowolny przycisk klawiatury zosta zwolniony.
			Przekazany zostaje take obiekt, ktry wywoa akcj - czyli this.
		**/
		void keyReleased(QKeyEvent* e, CustomInput* sender);
};

/**
	Klasa wykorzystana do oddzielenia listy uytkownikw od historii
	i pola do wpisywania wiadomoci
**/
class KaduSplitter : public QSplitter
{
	protected:
		QValueList<KaduTextBrowser*> textbrowsers;
		void drawContents(QPainter* p);
		void childEvent(QChildEvent* c);

	public:
		KaduSplitter(QWidget* parent = 0, const char* name = 0);
		KaduSplitter(Orientation o, QWidget* parent = 0, const char* name = 0);
};

class QMimeSourceFactory;
/**
	Okno rozmowy
**/
class Chat : public QWidget
{
	Q_OBJECT

	private:
		friend class ChatManager;
		struct RegisteredButton
		{
			QString name;
			QObject* receiver;
			QString slot;
		};
		static QValueList<RegisteredButton> RegisteredButtons;
		QMap<QString,QPushButton*> Buttons;
		
		QValueList<ChatMessage *> ChatMessages;
	
		UinsList Uins;
		int index;
		QString title_buffer;
		QTimer* title_timer;  
		QColor actcolor;
		
		QMimeSourceFactory *bodyformat;
		EmoticonSelector* emoticon_selector;
		ColorSelector* color_selector;
		QPushButton* boldbtn;
		QPushButton* italicbtn;
		QPushButton* underlinebtn;
		QPushButton* colorbtn;
		QPushButton* iconsel;
		QPushButton* autosend;
		QPushButton* lockscroll;
		QAccel* acc;
		QPushButton* sendbtn;
		UserBox* userbox;
		QString myLastMessage;
		int myLastFormatsLength;
		void* myLastFormats;
		int seq;
		KaduSplitter *vertSplit, *horizSplit;
		int ParagraphSeparator;
		QDateTime lastMsgTime;

		void pruneWindow();

	private slots:
		void userWhois();
		void emoticonSelectorClicked();
		void changeColor();
		void addMyMessageToHistory();
		void clearChatWindow();
		void pageUp();
		void pageDown();
		void insertImage();
		void imageReceivedAndSaved(UinType sender,uint32_t size,uint32_t crc32,const QString& path);

		void connectAcknowledgeSlots();
		void disconnectAcknowledgeSlots();

	protected:
		virtual void closeEvent(QCloseEvent*);
		QString convertCharacters(QString text, const QColor &bgcolor, EmoticonsStyle style);
		virtual void windowActivationChange(bool oldActive);
		virtual void keyPressEvent(QKeyEvent* e);
		virtual void mouseReleaseEvent(QMouseEvent *e);

	public:
		// FIXME - nie powinno byc publicznych zmiennych
		KaduTextBrowser* body;
		CustomInput* edit;
		QHBox* buttontray;
		/**
			Rejestruje opcje modulu Chat w oknie konfiguracji.
		**/
		static void initModule();
		Chat(UinsList uins, QWidget* parent = 0, const char* name = 0);
		~Chat();
		/**
			Rejestruje przycisk w oknie chat, ustawia funkcje do obsugi 
			sygnau "clicked"
			@param name nazwa przycisku aby moc si do niego odwoa
			@param receiver obiekt ktry zawiera funkcje do obsugi zdarzenia
			@param slot funkcja ktra bdzie obsugiwa zdarzenie "clicked"
		**/
		static void registerButton(const QString& name, QObject* receiver, const QString& slot);
		/**
			Wyrejestrowuje przycisk ze wszystkich okien
			@param name nazwa przycisku do wyrejestrowania
		**/
		static void unregisterButton(const QString& name);
		/**
			Zwraca wskanik do przycisku 
			@param name nazwa przycisku
			@return wskanik do przycisku, jeli przycisk nie istnieje
			zwraca NULL
		**/
		QPushButton* button(const QString& name);
		/**
			Formatuje wszystkie wiadomoci na html'a,
			dodaje kolory ta i czcionek
			@param msgs lista wiadomoci do sformatowania
		**/
		void formatMessages(QValueList<ChatMessage *> &msgs);
		/**
			Formatuje wiadomo na html'a
			@param msg wiadomo ktra bdzie sformatowana
			@param myBgColor kolor ktry bdzie ustawiony jako to
			w naszych wiadomociach
			@param usrBgColor kolor, ktry bdzie ustawiony jako to
			wiadomoci naszego rozmwcy
			@param myFontColor kolor naszej czcionki
			@param usrFontColor kolor czcionki naszego rozmwcy
			@param style styl ikonek 
		**/
		void formatMessage(ChatMessage &msg,
				QColor myBgColor=QColor(), QColor usrBgColor=QColor(),
				QColor myFontColor=QColor(), QColor usrFontColor=QColor(),
				EmoticonsStyle style=(EmoticonsStyle)config_file.readNumEntry("Chat","EmoticonsStyle"));
		/**
			Dodaje wiadomo do okna
		**/
		void checkPresence(UinsList, const QString&, time_t, QValueList<ChatMessage *> &messages);
		/**
			Wpisuje do okna histori rozmowy
			@param senders lista uytkownikw identyfikujcych okno
			@param time ..
		**/
		void writeMessagesFromHistory(UinsList, time_t);
		/**
			Zwraca list numerw rozmowcw.
		**/
		const UinsList& uins();
		/**
			Daje dostp do wiadomoci aktualnie przechowywanych
			w oknie chat. Metody tej mona uy do zmiany treci
			lub waciwoci ktrej z wiadomoci w odpowiedzi
			na jakie zdarzenie.
		**/
		QValueList<ChatMessage*>& chatMessages();
		/**
			Odwiea zawarto okna uwzgldniajc ewentualne
			zmiany dokonane w ktrej wiadomoci z listy
			uzyskanej za pomoc metody chatMessages(),
			dodanie nowych wiadomoci lub usunicie istniejcych.
		**/
		void repaintMessages();
		/**
			Zwraca aktualny tytu okna
		**/
		const QString& title();

	public slots:
		/**
			Slot zmieniajcy kolory i czcionki w oknie
		**/
		void changeAppearance();
		/**
			Slot ustawiajcy tytu okna zgodnie z konfiguracj
		**/
		void setTitle();
		/**
			Slot dodaje ikonke do pola wpisywania wiadomoci
			@param emot ikonka np. <lol>
		**/
		void addEmoticon(QString);
		/**
			Slot dodaj wiadomoci do okna
			@param messages lista wiadomoci
		**/
		void scrollMessages(const QValueList<ChatMessage *> &);
		/**
			Slot informujcy o nowej wiadomoci 
			mruganie jeli okno jest nieaktywne
		**/
		void alertNewMessage();
		/**
			Slot wywoywany po naciniciu przycisku historii
		**/
		void HistoryBox();
		/**
			Slot wywoywany po naciniciu przycisku do wysyania wiadomoci
		**/
		void sendMessage();
		/**
			Slot wywoywany po nacinieciu przycisku anulowania
			wysyaniu wiadomoci
		**/
		void cancelMessage();
		/**
			Slot wpisujcy wiadomo do okna
			@see sendMessage
		**/
		void writeMyMessage();
		/**
		    	Slot zmieniajcy tytu 
		    	@see setTitle
		**/
		void changeTitle();
		/**
			Slot wywoywany przy naciniciu przycisku pogrubienia
		**/
		void toggledBold(bool on);
		/**
			Slot wywoywany przy naciniciu przycisku kursywy
		**/
		void toggledItalic(bool on);
		/**
			Slot wywoywany przy naciniciu przycisku podkrelenia
		**/
		void toggledUnderline(bool on);
		/**
			Slot wywoywany kiedy pozycja kursosa zmienia si
		**/
		void curPosChanged(int para, int pos);
		/**
			Slot zostaje wywoany gdy nacinito specjalny klawisz (skrt)
			odpowiadajcy np KEY_BOLD
		**/
		void specialKeyPressed(int key);
		/**
			Slot zostaje wywowany gdy uytkownik zmieni kolor czcionki
			ktr bdzie pisa
		**/
		void colorChanged(const QColor& color);
		/**
			Slot zostaje wywoany przy zamykaniu okna wyboru ikonek
		**/
		void colorSelectorAboutToClose();
		/**
			Slot zostaje wywoany gdy nasza wiadomo zostaa zablokowana
			przez serwer
		**/
		void messageBlockedSlot(int seq, UinType uin);
		/**
			Slot zostaje wywoany gdy skrzynka na wiadomoci
			zostaa zapeniona
		**/
		void messageBoxFullSlot(int seq, UinType uin);
		/**
			Slot zostaje wywoany gdy wiadomo nie zostaa dostarczona
		**/
		void messageNotDeliveredSlot(int seq, UinType uin);
		/**
			Slot zostaje wywoany gdy wiadomo zostaa poprawnie wysana
		**/
		void messageAcceptedSlot(int seq, UinType uin);
		
	signals:
		/**
			Sygnal jest emitowany gdy uzytkownik wyda polecenie
			wyslania wiadomosci, np klikajac na guzik "wyslij".
		**/
		void messageSendRequested(Chat* chat);
		/**
			Sygnal daje mozliwosc operowania na wiadomoci
			ktora ma byc wyslana do serwera juz w jej docelowej
			formie po konwersji z unicode i innymi zabiegami.
			Tresc wiadomosci mozna zmienic podmieniajac wskaznik
			msg na nowy bufor i zwalniajac stary (za pomoca free).
			Mozna tez przerwac dalsza jej obrobke ustawiajac
			wskaznik stop na true.
		**/
		void messageFiltering(const UinsList& uins, QCString& msg, bool& stop);
		/**
			Sygnal jest emitowany gdy zakonczy sie proces
			wysylania wiadomosci i zwiazanych z tym czynnosci.
			Oczywiscie nie wiemy czy wiadomosc dotarla.
		**/		
		void messageSent(Chat* chat);
};

class ColorSelectorButton : public QPushButton
{
	Q_OBJECT

	private:
		QColor color;

	private slots:
		void buttonClicked();

	public:
		ColorSelectorButton(QWidget* parent, const QColor& qcolor, int width=1, const char *name=0);

	signals:
		/**
			Sygna zostaje wysany po wybraniu koloru
		**/
		void clicked(const QColor& color);
};

class ColorSelector : public QWidget
{
	Q_OBJECT

	private slots:
		void iconClicked(const QColor& color);

	protected:
		void closeEvent(QCloseEvent*);

	public:
		ColorSelector(const QColor &defColor, QWidget* parent = 0, const char* name = 0);

	public slots:
		/**
		  	Slot wyrwnuje pozycj do widgeta
		**/
		void alignTo(QWidget* w);

	signals:
		/**
			Sygna jest emitowany gdy zamykany jest wybr kolorw
		**/
		void aboutToClose();
		/**
			Sygna jest emitowany gdy wybrano kolor
		**/
		void colorSelect(const QColor&);
};

class ChatSlots :public QObject
{
	Q_OBJECT

	private:
		void updatePreview();
	public:
		ChatSlots(QObject* parent=0, const char* name=0);

	public slots:
		/**
			Slot jest wywoywany gdy otwierane jest okno konfiguracji
		**/
		void onCreateConfigDialog();
		/**
			Slot jest wywoywany gdy zamykane jest okno konfiguracji
		**/
		void onDestroyConfigDialog();
		/**
			Slot jest wywoywany gdy wybierany jest styl ikonek
		**/
		void chooseEmoticonsStyle(int index);
		/**
			Slot jest wywoywany gdy wybierane jest czy otwierane okno
			ma by pusty czy nie
		**/
		void onPruneChat(bool toggled);
		/**
		**/
		void onFoldLink(bool toggled);
		/**
			Slot jest wywoywany gdy wybierany jest maksymalny czas
			blokowania zamknicia okna po otrzymaniu wiadomoci
		**/
		void onBlockClose(bool toggled);
		/**
			Slot jest wywoywany gdy zmieniono kolor,
			aby uaktualni podgld 
		**/
		void chooseColor(const char* name, const QColor& color);
		/**
			Slot jest wywoywany gdy zmieniono czcionke,
			aby uaktualni podgld 
		**/
		void chooseFont(const char* name, const QFont& font);
		/**
			Slot jest wywoywany gdy wybrano z listy przegldark
		**/
		void findAndSetWebBrowser(int selectedBrowser);
		/**
			Slot jest wywoywany gdy aktywowane pole do wpisania opcji
		**/
		void findAndSetBrowserOption(int selectedOption);
		/**
			Slot ustawia przegldarki wraz z opcjami
		**/
		static void initBrowserOptions(QComboBox *browserCombo, QComboBox *browserOptionsCombo, QLineEdit *browserPath);
		/**
			Slot jest wywoywany gdy wybrano z listy przegldark
		**/
		static void findBrowser(int selectedBrowser, QComboBox *browserCombo, QComboBox *browserOptionsCombo, QLineEdit *browserPath);
		/**
			Slot ustawia przegldarki wraz z opcjami
		**/
		static void setBrowserOption(int selectedOption, QLineEdit *browserPathEdit, int chosenBrowser);
};

#endif
